import blessings
import sys
import os.path

from pysoup.display._display import Display, DisplayPipe

import pysoup.utils
import pysoup.utils.assets

class InteractiveDisplay(Display):

    graphical_assets_directory = os.path.join(os.path.dirname(__file__), '../../assets')

    def __init__(self):
        super(InteractiveDisplay, self).__init__()
        self._blessings = blessings.Terminal()
        self._lines = []

        self._clear_screen()
        self._print_logo()

    def create_pipe(self):
        key = len(self._lines)
        self._lines.append('')
        print ''
        self._refresh()
        return DisplayPipe(self, key)

    def update(self, key, message, type):
        self._lines[key] = self._format_message(message, type)
        self._refresh()

    def _refresh(self):
        output = self._clean()

        for line in self._lines:
            output += line + '\n'

        print output,

    def _format_message(self, message, type):
        max_line_length = sys.maxint
        message = '{0}'.format(str(message)[:max_line_length])
        if type == 'success':
            return self._blessings.green(message)
        elif type == 'error':
            return self._blessings.red(message)
        elif type == 'notify':
            return self._blessings.bright_yellow(message)
        else:
            return self._blessings.white(message)

    def _clear_screen(self):
        print self._blessings.clear

    def _clean(self):
        return (self._blessings.move_up * len(self._lines)) + self._blessings.clear_eos

    def _print_logo(self):
        print pysoup.utils.assets.LOGO



#!/usr/bin/env python
import argparse
import argcomplete
import os
import getpass

import pysoup


def main():

    parser = argparse.ArgumentParser()

    parser.add_argument('-q', '--quiet', help='no output to screen', action='store_true')

    commands = parser.add_subparsers(dest='command')

    cmd_install = commands.add_parser('install', help='install dependencies for a python project')
    cmd_install.add_argument('-g', '--global-installation', help='install dependencies as global', action='store_true')
    cmd_install.add_argument('-r', '--require-custom-file', help='use custom yaml file', default='soup.yaml')

    cmd_init = commands.add_parser('init', help='create a new yaml file for your project')
    cmd_init.add_argument('-n', '--name', help='project name', required=True)
    cmd_init.add_argument('-v', '--version', help='project version', required=False, default='0.0.1')
    cmd_init.add_argument('-a', '--author', help='project author', required=False, default=getpass.getuser())
    cmd_init.add_argument('-r', '--repository', help='project repository', required=False, default='')

    cmd_add = commands.add_parser('add', help='add a dependency to the yaml file')
    cmd_add.add_argument('name', help='name of the dependency')
    cmd_add.add_argument('version', help='version of the dependency')

    cmd_set = commands.add_parser('set', help='set a property in the yaml file')
    cmd_set.add_argument('-n', '--name', help='project name', required=False, default='')
    cmd_set.add_argument('-v', '--version', help='project version', required=False, default='')
    cmd_set.add_argument('-a', '--author', help='project author', required=False, default='')
    cmd_set.add_argument('-r', '--repository', help='project repository', required=False, default='')

    argcomplete.autocomplete(parser)
    args = parser.parse_args()

    if args.command == 'install':
        cwd = os.getcwd()
        target_file = args.require_custom_file
        is_global = args.global_installation
        is_quiet = args.quiet
        pysoup.PySoup.start_with_args('install', cwd, target_file, is_global, is_quiet)

    if args.command == 'init' or args.command == 'set':
        cwd = os.getcwd()
        target_file = 'soup.yaml'
        is_global = False
        is_quiet = args.quiet
        custom_configuration = {'name': args.name,
                                'version': args.version,
                                'author': args.author,
                                'repository': args.repository}

        if custom_configuration.values() == ['' for i in xrange(len(custom_configuration))]:
            print 'at least one argument is required.\nsee: soup {0} -h'.format(args.command)
        else:
            pysoup.PySoup.start_with_args(args.command,
                                          cwd,
                                          target_file,
                                          is_global,
                                          is_quiet,
                                          custom_configuration=custom_configuration)

    if args.command == 'add':
        cwd = os.getcwd()
        target_file = 'soup.yaml'
        is_global = False
        is_quiet = args.quiet
        custom_configuration = {'dependencies': {args.name: args.version}}
        pysoup.PySoup.start_with_args('add',
                                      cwd,
                                      target_file,
                                      is_global,
                                      is_quiet,
                                      custom_configuration=custom_configuration)

if __name__ == '__main__':
    main()



class Display(object):

    def __init__(self):
        pass

    def create_pipe(self):
        raise NotImplementedError()

    def update(self, key, message, type):
        raise NotImplementedError()


class DisplayPipe(object):

    def __init__(self, display, key):
        self._display = display
        self._key = key

    def log(self, message):
        self._update_display(message, 'log')

    def success(self, message):
        self._update_display(message, 'success')

    def error(self, message):
        self._update_display(message, 'error')

    def notify(self, message):
        self._update_display(message, 'notify')

    def _update_display(self, message, type):
        self._display.update(self._key, message, type)



import yaml


class Parser(object):

    def __init__(self):
        pass

    def parse_file(self, path):
        with open(path, 'rb') as f:
            yaml_data = f.read()
        return self.parse(yaml_data)

    def parse(self, string):
        return yaml.load(string)

    def dump_to_file(self, configuration, path):
        yaml_data = self.dump(configuration)
        with open(path, 'wb') as f:
            f.write(yaml_data)

    def dump(self, configuration):
        yaml_string = ''
        for key in ['name', 'author', 'version', 'repository', 'dependencies']:
            if configuration.has_key(key):
                yaml_string += yaml.dump({key: configuration[key]}, default_flow_style=False)
        return yaml_string



import os.path

from twisted.internet import defer, reactor

import pysoup.logger
import pysoup.display
import pysoup.parser
import pysoup.pip
import pysoup.utils
import pysoup.venv


class PySoup(object):

    def __init__(self, display, path, is_global):
        self._path = path
        self._is_global = is_global

        self._logger = pysoup.logger.Logger(path)

        self._parser = pysoup.parser.Parser()

        self._display = display
        self._display_pipes = {}
        self._add_display_pipe('sys')

        self._venv = None
        self._pip = None

    @defer.inlineCallbacks
    def install_from_file(self, target_config_file):
        self._display_pipes['sys'].log('Loading configuration file...')
        try:
            configuration = self._parser.parse_file(target_config_file)
        except:
            self._display_pipes['sys'].error('Could not load target file!')
            raise PysoupExecutionError()

        self._venv = self._create_venv()
        if not self._is_global:
            self._display_pipes['sys'].log('Setting up virtualenv...')
            try:
                yield self._venv.create()
            except:
                self._display_pipes['sys'].error('Could not setup virtualenv!')
                raise PysoupExecutionError()

        failed_dependencies = None
        if configuration.has_key('dependencies'):
            self._pip = self._create_pip()
            self._display_pipes['sys'].log('Installing pip dependencies...')
            _, failed_dependencies = yield self._pip.install_dependencies(configuration['dependencies'])
        else:
            self._display_pipes['sys'].log('No dependencies to install, skipping...')

        if not failed_dependencies:
            self._display_pipes['sys'].success('Done!')
        else:
            self._display_pipes['sys'].error('Done, but some dependencies were not installed. See soup.log')
            self._logger.log_dependency_results(failed_dependencies)

        self._logger.dump_to_file()
        yield 'completed'

    @defer.inlineCallbacks
    def init_new_project(self, configuration, target_config_file):
        if os.path.exists(target_config_file):
            self._display_pipes['sys'].notify('target file already exists, aborting...')
        else:
            yield self.set_project_attributes(configuration, target_config_file)

        yield 'completed'

    @defer.inlineCallbacks
    def set_project_attributes(self, configuration, target_config_file):
        new_config = {key: value for key, value in configuration.iteritems() if value}

        try:
            old_config = self._parser.parse_file(target_config_file)

            for key, value in old_config.iteritems():
                if not new_config.has_key(key):
                    new_config[key] = value
        except:
            pass

        self._parser.dump_to_file(new_config, target_config_file)
        self._display_pipes['sys'].success('soup configuration was updated successfuly!')

        yield 'completed'

    @defer.inlineCallbacks
    def add_dependency(self, configuration, target_config_file):
        try:
            target_config = self._parser.parse_file(target_config_file)

            try:
                target_dependencies = target_config['dependencies']
            except:
                target_dependencies = {}

            for dependency, version in configuration['dependencies'].iteritems():
                target_dependencies[dependency] = version

            target_config['dependencies'] = target_dependencies

            self._parser.dump_to_file(target_config, target_config_file)

            self._display_pipes['sys'].success('dependency added successfully!')
        except:
            self._display_pipes['sys'].notify('soup configuration does not exists, aborting...')

        yield 'completed'

    def _add_display_pipe(self, pipe_name):
        self._display_pipes[pipe_name] = self._display.create_pipe()
        return self._display_pipes[pipe_name]

    def _create_venv(self):
        if not self._is_global:
            return pysoup.venv.Virtualenv(self._add_display_pipe('venv'), self._path)
        else:
            return None

    def _create_pip(self):
        return pysoup.pip.Pip(self._add_display_pipe('pip'), venv=self._venv)

    @staticmethod
    def start_with_args(command, cwd, target_config_file, is_global=False, is_silent=False, custom_configuration=None):
        if is_silent:
            display = pysoup.display.DisplayAdapter.create_silent_display()
        else:
            display = pysoup.display.DisplayAdapter.create_interactive_display()

        soup = PySoup(display, cwd, is_global)

        exec_command = None

        if command == 'install':
            file_path = os.path.join(cwd, target_config_file)
            exec_command = soup.install_from_file(file_path)

        elif command == 'init':
            file_path = os.path.join(cwd, target_config_file)
            exec_command = soup.init_new_project(custom_configuration, file_path)

        elif command == 'set':
            file_path = os.path.join(cwd, target_config_file)
            exec_command = soup.set_project_attributes(custom_configuration, file_path)

        elif command == 'add':
            file_path = os.path.join(cwd, target_config_file)
            exec_command = soup.add_dependency(custom_configuration, file_path)


        exec_command.addCallbacks(PySoup._on_soup_callback, PySoup._on_soup_errback)
        reactor.run()

    @staticmethod
    def _on_soup_callback(result):
        PySoup._teardown(0)

    @staticmethod
    def _on_soup_errback(error):
        PySoup._teardown(1)

    @staticmethod
    def _teardown(exit_code):
        reactor.callFromThread(reactor.stop)


class PysoupExecutionError(Exception):
    pass



from twisted.internet import defer

import pysoup.utils


class Pip(object):

    def __init__(self, display_pipe, venv=None):
        self._display_pipe = display_pipe
        self._venv = venv

    @defer.inlineCallbacks
    def install_dependencies(self, dependency_map):
        failed_installations = []
        successful_installations = []
        dependency_count = len(dependency_map)
        current_dependency = 0

        for dependency, version in dependency_map.iteritems():
            current_dependency += 1
            self._display_pipe.log('installing {0} ({1}/{2})'.format(dependency, current_dependency, dependency_count))

            try:
                yield self._install_dependency(dependency, version)
                successful_installations.append(dependency)
            except:
                failed_installations.append(dependency)

        if failed_installations:
            self._display_pipe.error('failed to install {0} dependencies.'.format(len(failed_installations)))
        else:
            self._display_pipe.success('all dependencies installed successfully!')

        defer.returnValue((successful_installations, failed_installations))

    @defer.inlineCallbacks
    def _install_dependency(self, dependency, version=''):
        command = 'pip install "{0}"'.format(self._get_pip_complient_string(dependency, version))

        if self._venv:
            success = yield self._venv.execute_in_venv(command)
        else:
            success = yield pysoup.utils.execute_shell_command(command)

        if success != 0:
            raise Exception('could not install dependency')

    def _get_pip_complient_string(self, dependency, version=''):
        complient_dependency = dependency

        if version[0] == '*':
            complient_version = ''
        elif version[0] == '+':
            complient_version = '>={0}'.format(version[1:])
        elif version[0] == '-':
            complient_version = '<={0}'.format(version[1:])
        else:
            complient_version = '=={0}'.format(version)

        return '{0}{1}'.format(complient_dependency, complient_version)



import pysoup.display.interactive_display
import pysoup.display.silent_display


class DisplayAdapter(object):

    @staticmethod
    def create_interactive_display():
        return pysoup.display.interactive_display.InteractiveDisplay()

    @staticmethod
    def create_silent_display():
        return pysoup.display.silent_display.SilentDisplay()



import os.path

import pysoup.utils.assets

class Logger(object):

    def __init__(self, cwd):
        self._log = ''
        self._cwd = cwd

    def log(self, text):
        self._log += '{0}\n'.format(text)

    def log_dependency_results(self, failed_dependencies):
        for dependency in failed_dependencies:
            self.log('could not install {0}'.format(dependency))

    def dump_to_file(self, filename='soup.log'):
        if self._log != '':
            with open(os.path.join(self._cwd, filename), 'wb') as f:
                f.write(pysoup.utils.assets.LOGO)
                f.write('\n{0}'.format(self._log))



from pysoup.display._display import Display, DisplayPipe


class SilentDisplay(Display):

    def __init__(self):
        super(SilentDisplay, self).__init__()

    def create_pipe(self):
        return DisplayPipe(self, '')

    def update(self, key, message, type):
        pass



#!/usr/bin/env python
from setuptools import setup, find_packages


setup(name='pysoup',
      version='0.1.1',
      description='Package management in a bowl',
      author='Roy Sommer',
      url='https://www.github.com/illberoy/pysoup',
      download_url='https://github.com/illberoy/pysoup/tarball/0.1.1',
      packages=find_packages(),
      include_package_data=True,
      scripts=['soup.py'],
      install_requires=['Twisted==15.5.0', 'blessings==1.6.0', 'PyYAML==3.11', 'argcomplete==1.0.0'],
      entry_points={'console_scripts': ['soup = soup:main']})



import os.path

from twisted.internet import defer

import pysoup.utils


class Virtualenv(object):

    def __init__(self, display_pip, path):
        self._display_pipe = display_pip
        self._path = path

    @property
    def path(self):
        return self._path

    @property
    def venv_path(self):
        return os.path.join(self._path, 'venv')

    @property
    def source_path(self):
        return os.path.join(self.venv_path, 'bin/activate')

    @defer.inlineCallbacks
    def create(self):
        self._display_pipe.log('Ensuring virtualenv environment at {0}'.format(self._path))

        code = yield pysoup.utils.execute_shell_command('mkdir -p {0} && virtualenv --no-site-packages -q {0}'.format(self.venv_path))
        if code != 0:
            self._display_pipe.error('Failed to setup virtualenv at target! ({0})'.format(self._path))
            raise Exception('Could not create virtualenv')

        self._display_pipe.notify('Virtualenv is ready')

    @defer.inlineCallbacks
    def execute_in_venv(self, command):
        code = yield pysoup.utils.execute_shell_command('source {0} && {1}'.format(self.source_path, command))
        defer.returnValue(code)



from twisted.internet import defer, utils


@defer.inlineCallbacks
def execute_shell_command(command):
    s, e, code = yield utils.getProcessOutputAndValue('/bin/bash',
                                                      args=['-c', command])
    defer.returnValue(code)
